from __future__ import absolute_import, with_statement

from django.core.exceptions import ImproperlyConfigured

from hosts.reverse import get_hostconf_module, get_host_patterns
from hosts.tests.base import override_settings, HostsTestCase

class ReverseTest(HostsTestCase):
    pass


class UtilityTests(HostsTestCase):

    @override_settings(ROOT_HOSTCONF='hosts.tests.hosts.simple')
    def test_get_hostconf_module(self):
        from hosts.tests.hosts import simple
        self.assertEqual(get_hostconf_module(), simple)

    def test_get_hostconf_module_no_default(self):
        from hosts.tests.hosts import simple
        self.assertEqual(get_hostconf_module('hosts.tests.hosts.simple'), simple)

    def test_get_host_patterns(self):
        self.assertRaisesWithMessage(ImproperlyConfigured,
            'Missing ROOT_HOSTCONF setting', get_host_patterns)

        with self.settings(ROOT_HOSTCONF='hosts.tests.hosts'):
            self.assertRaisesWithMessage(ImproperlyConfigured,
                "Missing host_patterns in 'hosts.tests.hosts'", get_host_patterns)

        with self.settings(ROOT_HOSTCONF='hosts.tests.hosts.simple'):
            from hosts.tests.hosts import simple
            self.assertEqual(get_host_patterns(), simple.host_patterns)



from __future__ import absolute_import, with_statement

from django.conf import settings, UserSettingsHolder
from django.core.handlers.wsgi import WSGIRequest
from django.core.handlers.base import BaseHandler
from django.test import TestCase, Client
from django.utils.functional import wraps

from ..reverse import clear_host_caches

class override_settings(object):
    """
    Acts as either a decorator, or a context manager.  If it's a decorator it
    takes a function and returns a wrapped function.  If it's a contextmanager
    it's used with the ``with`` statement.  In either event entering/exiting
    are called before and after, respectively, the function/block is executed.
    """
    def __init__(self, **kwargs):
        self.options = kwargs
        self.wrapped = settings._wrapped

    def __enter__(self):
        self.enable()

    def __exit__(self, exc_type, exc_value, traceback):
        self.disable()

    def __call__(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            with self:
                return func(*args, **kwargs)
        return inner

    def enable(self):
        override = UserSettingsHolder(settings._wrapped)
        for key, new_value in self.options.items():
            setattr(override, key, new_value)
        settings._wrapped = override

    def disable(self):
        settings._wrapped = self.wrapped


# Adapted from Simon Willison's snippet: http://djangosnippets.org/snippets/963/.
class RequestFactory(Client):
    """
    Class that lets you create mock Request objects for use in testing.

    Usage:

    rf = RequestFactory()
    get_request = rf.get('/hello/')
    post_request = rf.post('/submit/', {'foo': 'bar'})

    This class re-uses the django.test.client.Client interface, docs here:
    http://www.djangoproject.com/documentation/testing/#the-test-client

    Once you have a request object you can pass it to any view function,
    just as if that view had been hooked up using a URLconf.

    """
    def request(self, **request):
        """
        Similar to parent class, but returns the request object as soon as it
        has created it.
        """
        environ = {
            'HTTP_COOKIE': self.cookies,
            'PATH_INFO': '/',
            'QUERY_STRING': '',
            'REQUEST_METHOD': 'GET',
            'SCRIPT_NAME': '',
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': 80,
            'SERVER_PROTOCOL': 'HTTP/1.1',
        }
        environ.update(self.defaults)
        environ.update(request)
        request = WSGIRequest(environ)

        handler = BaseHandler()
        handler.load_middleware()
        for middleware_method in handler._request_middleware:
            if middleware_method(request):
                raise Exception("Couldn't create request object - "
                                "request middleware returned a response")
        return request

class HostsTestCase(TestCase):

    def setUp(self):
        # Every test needs access to the request factory.
        self.factory = RequestFactory()
        self.old_root_urlconf = getattr(settings, 'ROOT_URLCONF', '')
        settings.ROOT_URLCONF = 'hosts.tests.urls.root'

    def tearDown(self):
        settings.ROOT_URLCONF = self.old_root_urlconf
        clear_host_caches()

    def assertRaisesWithMessage(self, error, message, callable, *args, **kwargs):
        self.assertRaises(error, callable, *args, **kwargs)
        try:
            callable(*args, **kwargs)
        except error, e:
            self.assertEqual(message, str(e))

    def settings(self, **kwargs):
        """
        A context manager that temporarily sets a setting and reverts
        back to the original value when exiting the context.
        """
        return override_settings(**kwargs)



import re
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import get_callable, get_mod_func
from django.utils.datastructures import SortedDict
from django.utils.encoding import smart_str

def patterns(prefix, *args):
    hosts = SortedDict()

    for arg in args:
        if isinstance(arg, (list, tuple)):
            arg = host(prefix=prefix, *arg)
        elif isinstance(arg, host):
            arg.add_prefix(prefix)
        name = arg.name
        if name in hosts:
            raise ImproperlyConfigured("Duplicate host name: %s" % name)
        if name == 'default':
            raise ImproperlyConfigured("Reserved host name: %s" % name)
        hosts[name] = arg
    return hosts

class host(object):

    def __init__(self, regex, urlconf, name, callback=None, prefix=''):
        # Compile hosts. We add a literal fullstop to the end of every
        # pattern to avoid rather unwieldy escaping in every definition.
        self.regex = regex
        self.compiled_regex = re.compile(r'%s(\.|$)' % regex)
        if prefix:
            urlconf = prefix + '.' + urlconf
        self.urlconf = urlconf
        self.name = name
        if callable(callback):
            self._callback = callback
        else:
            self._callback, self._callback_str = None, callback

    @property
    def callback(self):
        if self._callback is not None:
            return self._callback
        if self._callback_str is None:
            return lambda *args, **kwargs: None
        try:
            self._callback = get_callable(self._callback_str)
        except ImportError, e:
            mod_name, _ = get_mod_func(self._callback_str)
            raise ImproperlyConfigured(
                "Could not import %s. Error was: %s" % (mod_name, str(e)))
        except AttributeError, e:
            mod_name, func_name = get_mod_func(self._callback_str)
            raise ImproperlyConfigured(
                "Tried %s in module %s. Error was: %s" % (func_name, mod_name, str(e)))
        return self._callback

    def __repr__(self):
        return smart_str(u'<%s %s %s>'
                         % (self.__class__.__name__, self.name, self.regex))

    def add_prefix(self, prefix):
        """
        Adds the prefix string to a string-based callback.
        """
        if not prefix or not hasattr(self, '_callback_str'):
            return
        self._callback_str = prefix + '.' + self._callback_str



from django.http.utils import fix_location_header
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.core.handlers.base import BaseHandler

from debug_toolbar.panels import DebugPanel


class HostPanel(DebugPanel):
    """
    Panel that allows you to alter the host you are viewing the site
    without /etc/hosts hacks.
    """

    name = 'Host'
    has_content = True

    def nav_title(self):
        return _("Host")

    def nav_subtitle(self):
        return self.host

    def url(self):
        return ''

    def title(self):
        return _("Host navigation")

    def content(self):
        context = self.context.copy()
        context['host'] = self.host
        return render_to_string('hosts/panel.html', context)

    def process_request(self, request):
        self.host = request.COOKIES.get('_host')

        request.META.pop('HTTP_HOST', '')
        if self.host:
            request.META['HTTP_HOST'] = self.host

            # django.http.utils.fix_location_header rewrites any relative
            # Location header to an absolute one. For example:
            #
            #    Location: /foo   ==>   Location: http://sub.example.com/foo
            #
            # This causes problems when testing hosts locally so we remove
            # it here.

            try:
                BaseHandler.response_fixes.remove(fix_location_header)
            except ValueError:
                pass



from __future__ import absolute_import, with_statement

from django.template import Template, Context, TemplateSyntaxError

from hosts.tests.base import override_settings, HostsTestCase


class TemplateTagsTest(HostsTestCase):

    def render(self, template, context=None):
        if context is None:
            context = Context({})
        return Template(template).render(context).strip()

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='hosts.tests.hosts.simple')
    def test_host_url_tag_simple(self):
        rendered = self.render(
            "{% load hosts %}{% host_url simple-direct on www %}")
        self.assertEqual(rendered, '//www.example.com/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='hosts.tests.hosts.simple')
    def test_host_url_tag_with_args(self):
        rendered = self.render(
            "{% load hosts %}{% host_url simple-direct on with_args 'www.eggs.spam' %}")
        self.assertEqual(rendered, '//www.eggs.spam/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        PARENT_HOST='eggs.spam',
        ROOT_HOSTCONF='hosts.tests.hosts.simple')
    def test_host_url_tag_with_kwargs(self):
        rendered = self.render(
            "{% load hosts %}{% host_url simple-direct on with_kwargs username='johndoe' %}")
        self.assertEqual(rendered, '//johndoe.eggs.spam/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='hosts.tests.hosts.simple',
        PARENT_HOST='eggs.spam')
    def test_host_url_tag_parent_host(self):
        rendered = self.render(
            "{% load hosts %}{% host_url simple-direct on static %}")
        self.assertEqual(rendered, '//static.eggs.spam/simple/')

    @override_settings(
        DEFAULT_HOST='www',
        ROOT_HOSTCONF='hosts.tests.hosts.simple')
    def test_raises_template_syntaxerror(self):
        self.assertRaises(TemplateSyntaxError, self.render, "{% load hosts %}{% host_url %}")
        self.assertRaises(TemplateSyntaxError, self.render, "{% load hosts %}{% host_url simple-direct on %}")



from __future__ import absolute_import, with_statement

from django.core.exceptions import ImproperlyConfigured

from hosts import patterns, host
from hosts.reverse import get_host_patterns
from hosts.tests.base import HostsTestCase


class PatternsTests(HostsTestCase):

    def test_pattern(self):
        host_patterns = patterns('',
            host(r'api', 'api.urls', name='api'),
        )
        self.assertEqual(len(host_patterns), 1)
        self.assertTrue(isinstance(host_patterns['api'], host))

    def test_pattern_as_tuple(self):
        host_patterns = patterns('',
            (r'api', 'api.urls', 'api'),
        )
        self.assertEqual(len(host_patterns), 1)
        self.assertTrue(isinstance(host_patterns['api'], host))

    def test_pattern_with_duplicate(self):
        api_host = host(r'api', 'api.urls', name='api')
        self.assertRaises(ImproperlyConfigured, patterns, '', api_host, api_host)

    def test_pattern_with_default(self):
        default_host = host(r'www', 'mysite.urls', name='default')
        self.assertRaises(ImproperlyConfigured, patterns, '', default_host)


class HostTests(HostsTestCase):

    def test_host(self):
        api_host = host(r'api', 'api.urls', name='api')
        self.assertTrue(isinstance(api_host, host))

    def test_host_prefix(self):
        api_host = host(r'api', 'api.urls', name='api', prefix='spam.eggs')
        self.assertEqual(api_host.urlconf, 'spam.eggs.api.urls')

    def test_host_string_callback(self):
        api_host = host(r'api', 'api.urls', name='api', callback='hosts.reverse.get_host_patterns')
        self.assertEqual(api_host.callback, get_host_patterns)



from django.conf.urls.defaults import patterns, url

from debug_toolbar.urls import _PREFIX

urlpatterns = patterns('hosts.contrib.toolbar.views',
    url(r'^%s/host/redirect/$' % _PREFIX, 'redirect', name='hosts-debug-redirect'),
)



from __future__ import absolute_import

import re
from django import template
from django.conf import settings
from django.template import TemplateSyntaxError
from django.utils.encoding import smart_str

from hosts.reverse import reverse_crossdomain

register = template.Library()

kwarg_re = re.compile(r"(?:(\w+)=)?(.+)")

class HostURLNode(template.Node):

    @staticmethod
    def parse_args_kwargs(parser, bits):
        args, kwargs = [], {}
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError('Malformed arguments to host_url tag')
            name, value = match.groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))
        return args, kwargs

    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        if len(bits) < 2:
            raise TemplateSyntaxError("'%s' takes at least 1 argument" % bits[0])
        view = bits[1]
        bits = bits[1:] # Strip off view
        try:
            pivot = bits.index('on')
            try:
                host = bits[pivot+1]
            except IndexError:
                raise TemplateSyntaxError(
                    "'%s' arguments must include a host after 'on'" % bits[0])
            view_args, view_kwargs = cls.parse_args_kwargs(parser, bits[1:pivot])
            host_args, host_kwargs = cls.parse_args_kwargs(parser, bits[pivot+2:])

        except ValueError:
            # No "on <host>" was specified so use the default host
            host = settings.DEFAULT_HOST
            view_args, view_kwargs = cls.parse_args_kwargs(parser, bits[1:])
            host_args, host_kwargs = (), {}

        return cls(host, view, host_args, host_kwargs, view_args, view_kwargs)

    def __init__(self, host, view, host_args, host_kwargs, view_args, view_kwargs):
        self.host = host
        self.view = view
        self.host_args = host_args
        self.host_kwargs = host_kwargs
        self.view_args = view_args
        self.view_kwargs = view_kwargs

    def render(self, context):
        host_args = [x.resolve(context) for x in self.host_args]
        host_kwargs = dict((smart_str(k, 'ascii'), v.resolve(context))
                            for k, v in self.host_kwargs.iteritems())
        view_args = [x.resolve(context) for x in self.view_args]
        view_kwargs = dict((smart_str(k, 'ascii'), v.resolve(context))
                            for k, v in self.view_kwargs.iteritems())
        return reverse_crossdomain(self.host, self.view,
            host_args, host_kwargs, view_args, view_kwargs)


@register.tag
def host_url(parser, token):
    """
    Simple tag to reverse the URL inclusing a host.

    {% host_url url-name on host-name  %}
    {% host_url url-name on host-name 'spam' %}

    """
    return HostURLNode.handle_token(parser, token)



from __future__ import absolute_import

from hosts.defaults import patterns, host

host_patterns = patterns('',
    host(r'www\.example\.com', 'hosts.tests.urls.simple', name='www'),
    host(r'static', 'hosts.tests.urls.simple', name='static'),
    host(r'(\w+)', 'hosts.tests.urls.simple', name='with_args'),
    host(r'(?P<username>\w+)', 'hosts.tests.urls.simple', name='with_kwargs'),
)



from django.http import HttpResponseRedirect, HttpResponseBadRequest

from .forms import RedirectForm

def redirect(request):
    form = RedirectForm(request.GET)

    if not form.is_valid():
        return HttpResponseBadRequest(repr(form.errors))

    response = HttpResponseRedirect(form.cleaned_data['path'])
    response.set_cookie('_host', form.cleaned_data['host'])
    response.status_code = 307 # Re-submit POST requests

    return response



from os import path
from setuptools import setup, find_packages

README = path.abspath(path.join(path.dirname(__file__), 'README.rst'))

setup(
    name='django-hosts',
    description="Dynamic and static hosts support for Django.",
    long_description=open(README).read(),
    version='0.1.1',
    url='https://github.com/jezdez/django-hosts',
    author='Jannis Leidel',
    author_email='jannis@leidel.info',
    license='BSD',
    packages=find_packages(),
    package_data={
        'hosts': [
            'contrib/toolbar/templates/hosts/*.html',
        ],
    },
    install_requires = ['django >= 1.1, <1.4'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import set_urlconf

from hosts.reverse import get_host_patterns

class HostsMiddleware(object):
    """
    Adjust incoming request's urlconf based on hosts defined in
    settings.ROOT_HOSTCONF module.
    """
    def __init__(self):
        self.host_patterns = get_host_patterns()
        try:
            self.default_host = self.host_patterns[settings.DEFAULT_HOST]
        except AttributeError:
            raise ImproperlyConfigured("Missing DEFAULT_HOST setting")
        except KeyError:
            raise ImproperlyConfigured("Invalid DEFAULT_HOST setting")

    def process_request(self, request):
        request_host = request.get_host()
        # Find best match, falling back to settings.DEFAULT_HOST
        for host in self.host_patterns.itervalues():
            match = host.compiled_regex.match(request_host)
            if match:
                kwargs = match.groupdict()
                break
        else:
            host, kwargs = self.default_host, {}
        request.urlconf = host.urlconf
        try:
            set_urlconf(host.urlconf)
            return host.callback(request, **kwargs)
        finally:
            # Reset URLconf for this thread on the way out for complete
            # isolation of request.urlconf
            set_urlconf(None)



import re

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import NoReverseMatch, reverse
from django.http import QueryDict
from django.utils.encoding import force_unicode
from django.utils.functional import memoize
from django.utils.importlib import import_module
from django.utils.regex_helper import normalize

_hostconf_cache = {}
_host_patterns_cache = {}


def get_hostconf_module(hostconf=None):
    if hostconf is None:
        hostconf = settings.ROOT_HOSTCONF
    return import_module(hostconf)
get_hostconf_module = memoize(get_hostconf_module, _hostconf_cache, 1)


def get_host_patterns():
    try:
        hostconf = settings.ROOT_HOSTCONF
    except AttributeError:
        raise ImproperlyConfigured("Missing ROOT_HOSTCONF setting")
    hostconf_module = get_hostconf_module(hostconf)
    try:
        return hostconf_module.host_patterns
    except AttributeError:
        raise ImproperlyConfigured("Missing host_patterns in '%s'" % hostconf)
get_host_patterns = memoize(get_host_patterns, _host_patterns_cache, 0)


def clear_host_caches():
    global _hostconf_cache
    global _host_patterns_cache
    _hostconf_cache.clear()
    _host_patterns_cache.clear()


def reverse_host(name, args=None, kwargs=None):
    if args and kwargs:
        raise ValueError("Don't mix *args and **kwargs in call to reverse()!")

    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}

    host_patterns = get_host_patterns()
    try:
        host = host_patterns[name]
    except KeyError:
        raise NoReverseMatch("No host called %s exists" % name)

    unicode_args = [force_unicode(x) for x in args]
    unicode_kwargs = dict(((k, force_unicode(v)) for (k, v) in kwargs.iteritems()))

    for result, params in normalize(host.regex):
        if args:
            if len(args) != len(params):
                continue
            candidate = result % dict(zip(params, unicode_args))
        else:
            if set(kwargs.keys()) != set(params):
                continue
            candidate = result % unicode_kwargs

        if re.match(host.regex, candidate, re.UNICODE):
            return candidate

    raise NoReverseMatch("Reverse host for '%s' with arguments '%s' and "
                         "keyword arguments '%s' not found."
                         % (name, args, kwargs))

def reverse_crossdomain_part(host, path, host_args=None, host_kwargs=None):
    if host_args is None:
        host_args = ()
    if host_kwargs is None:
        host_kwargs = {}

    host_part = reverse_host(host, args=host_args, kwargs=host_kwargs)

    if getattr(settings, 'PARENT_HOST', False):
        host_part = '%s.%s' % (host_part, settings.PARENT_HOST.lstrip('.'))

    if getattr(settings, 'EMULATE_HOSTS', settings.DEBUG):
        query_string = QueryDict('', mutable=True)
        query_string.update({'host': host_part, 'path': path})
        redirect_path = reverse('hosts-debug-redirect')
        return '%s?%s' % (redirect_path, query_string.urlencode())

    return u'//%s%s' % (host_part, path)

def reverse_path(host, view, args=None, kwargs=None):
    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}
    host_patterns = get_host_patterns()
    try:
        urlconf = host_patterns[host].urlconf
    except KeyError:
        raise NoReverseMatch("No host called %s exists" % host)
    return reverse(view, args=args, kwargs=kwargs, urlconf=urlconf)

def reverse_crossdomain(host, view, host_args=None, host_kwargs=None,
        view_args=None, view_kwargs=None):
    path = reverse_path(host, view, view_args, view_kwargs)
    return reverse_crossdomain_part(host, path, host_args, host_kwargs)



from __future__ import absolute_import

from django.core.exceptions import ImproperlyConfigured

from hosts.middleware import HostsMiddleware
from hosts.tests.base import override_settings, HostsTestCase, RequestFactory

class MiddlewareTests(HostsTestCase):

    def test_missing_hostconf_setting(self):
        self.assertRaisesWithMessage(ImproperlyConfigured,
            'Missing ROOT_HOSTCONF setting', HostsMiddleware)

    @override_settings(ROOT_HOSTCONF='hosts.tests.hosts.simple')
    def test_missing_default_hosts(self):
        self.assertRaisesWithMessage(ImproperlyConfigured,
            'Missing DEFAULT_HOST setting', HostsMiddleware)

    @override_settings(
        ROOT_HOSTCONF='hosts.tests.hosts.simple',
        DEFAULT_HOST='boo')
    def test_wrong_default_hosts(self):
        self.assertRaisesWithMessage(ImproperlyConfigured,
            'Invalid DEFAULT_HOST setting', HostsMiddleware)

    @override_settings(
        ROOT_HOSTCONF='hosts.tests.hosts.simple',
        DEFAULT_HOST='www')
    def test_request_urlconf_module(self):
        rf = RequestFactory(HTTP_HOST='other.example.com')
        request = rf.get('/simple/')
        middleware = HostsMiddleware()
        middleware.process_request(request)
        self.assertEqual(request.urlconf, 'hosts.tests.urls.simple')
